// MacCapture.cpp
//
//  2001-2003 BEST Software Design -- All Rights Reserved
//
// Document Authors: Benjamin Schneider (BJS), Erick Tejkowski (EJT)
// Document Contributors: 
//
// Revision History
// April 1 2002 -- EJT & BJS (1)	Initial implementations for MacOS and Mac OS X

#include "capture.h"
#include <String.h> // BJS
//#include <PLStringFuncs.h>
#include <IsochronousDataHandler.h> // for SendPlayCommand
#include <DeviceControl.h> // for SendPlayCommand



extern struct REALcontrol captureControl;

REALevent captureEvents[] = {
	{ "Error(theError As Integer)" },
	{ "Record()" },
	{ "Stop()" }
};

struct captureData
{
	int 				fillColor;			// color in RB color format (xxRRGGBB)
	SeqGrabComponent 	theSGComp;
	GrafPtr 			theSGGraphicsPort; // BJS
	QT::Rect 			theSGBounds, theNaturalBounds;
	SGChannel 			theSGVideoChannel, theSGSoundChannel;
	Boolean				enableSound, enableVideo, pauseState, recordState, previewState, previewWhileRecording;
	Boolean				fastGrabPicture;
	//EventRecord 		captureEvent;
	Fixed				sampleRate, frameRate;
	short				sampleSize, numChan, theDepth, soundVolume;
	OSType				soundCType, videoCType;
	long				inDevNum, theKeyFrameRate;
	REALpicture			theNoPreviewPicture;
	CodecQ				theSpatialQuality, theTemporalQuality;
	SGDeviceList		theSGSoundDeviceList, theSGVideoDeviceList;
	ComponentInstance	theVideoComponent;
	

	//WINDOWS DATA MEMBERS
	char 				szCaptureFile[2048];
    short 				wCapFileSize;  // size in Meg
    long 				pBuilder;
    long 				pVW;
    long 				pME;
    long 				pASC;      // for audio cap
    long 				pVSC;      // for video cap
    long 				pGrabberBase;
    long 				pGrabber;
    long		 		pSmartTee;
    long				pNULLRenderer;
    long				dvGrabDecoder; // BJS 11/10/03
    long				dvPreviewDecoder; // BJS 11/10/03
    long				dvGrabSplitter; // BJS 11/10/03
    long				dvPreviewSplitter; // BJS 11/10/03
    long				aviDecompressor; // BJS 11/10/03
    long				colorSpaceGrabConverter; // BJS 11/10/03
    long				colorSpacePreviewConverter; // BJS 11/10/03
    long 				pMC;
    long 				pRender;
    long 				pVCap, pACap;
    long 				pFg;
    long 				pConfigAviMux;
    int  				iMasterStream;
    Boolean 			fPreviewing;
    Boolean 			fCapturing;
    Boolean 			fMPEG2;
    Boolean 			fCapAudio;
    Boolean 			fCapCC;
    double 				FrameRate;
    char 				wachFriendlyName[120];
    char 				wachAudioFriendlyName[120];
    Boolean 			fUseFrameRate;
	long 				ghwndApp;
	Boolean 			usingVMR;
	long				m_dwGraphRegister;
	
    long				videoSlotNumber;
    long				audioSlotNumber;
    
	long 				pMux;
    long 				pWrite;
    
    long				VRCreated;
    
    long				pInfTee;
    
    Boolean				useDVAudioDevice;
    
    long 				pDVSplit; 
	long 				pDVDec;
	Boolean				refreshPreviewPicture;
    long				pAudioRender;
    
};



static void captureInit(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	long 		result;
	Boolean 	gQuicktime41Installed = false;
	const char 	*myMessage;
	int 		msgLength;
	
	data->sampleRate = UnsignedFixedMulDiv(44100, 65536, 1);
	data->sampleSize = 16;
	data->numChan = 2;
	
	if (REALinRuntime())
		{
		OSErr error = Gestalt(gestaltQuickTime, &result);
		if (error == QT::noErr)
			{
			if (result >= 0x4100000)
				gQuicktime41Installed = true;
			}
	
		if (!gQuicktime41Installed)
			{
			myMessage = "QuickTime 4.1 or higher must be installed in order to use the Capture Control.";
			msgLength = 78;
			SysBeep(30);
			REALMessageBox(REALBuildString(myMessage, msgLength));
			return;
			}

		if (!REALenterMovies())
			{
			myMessage = "Please make sure QuickTime is properly installed on your computer.";
			msgLength = 66;
			SysBeep(30);
			REALMessageBox(REALBuildString(myMessage, msgLength));
			return;
			}
		}
}

static void captureOpen(REALcontrolInstance instance)
{
	
	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		captureStartup(instance);
		}
	else
		REALSetControlVisible(instance, 1L); // BJS
	
}

static void captureClose(REALcontrolInstance instance)
{
	
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	if (data->theSGSoundChannel)
		{
		if (data->theSGSoundDeviceList != nil)
			{
			err = SGDisposeDeviceList(data->theSGComp, data->theSGSoundDeviceList);
			data->theSGSoundDeviceList = nil;
			}
		err = SGDisposeChannel(data->theSGComp, data->theSGSoundChannel);
		data->theSGSoundChannel = nil;
		}
	if (data->theSGVideoChannel)
		{
		if (data->theSGVideoDeviceList != nil)
			{
			err = SGDisposeDeviceList(data->theSGComp, data->theSGVideoDeviceList);
			data->theSGVideoDeviceList = nil;
			}
		err = SGDisposeChannel(data->theSGComp, data->theSGVideoChannel);
		data->theSGVideoChannel = nil;
		}

	if (data->theSGComp)
		{
		CloseComponent(data->theSGComp);
		data->theSGComp = nil;
		}
}


static void captureStartup(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	OSErr 		err = QT::noErr;
	
	if (REALinRuntime())
	{
	// comment out for non demo build
	//SysBeep(30);
	//REALMessageBox(REALBuildString("This is a demo of BEST Software Design's BESTCapture plugin for REALbasic.  Please purchase it to remove this warning.",118));
	// end commnent out for non demo build	
	data->theSGComp = OpenDefaultComponent(SeqGrabComponentType,0);

	if (!data->theSGComp)
		return;

  #if TARGET_WIN32

  #else
	REALRegisterEventFilter (captureEventFilterHandler, (long) instance);
	#endif
		
	REALGetControlBounds(instance, &data->theSGBounds);
	
	err = SGInitialize(data->theSGComp);
		if (err != QT::noErr)
			{
			CloseComponent(data->theSGComp);
			data->theSGComp = nil;
			captureFireErrorEvent(instance, err);
			return;
			}

		WindowPtr theSGWindowPtr = REALGetWindowHandle(REALGetControlWindow(instance));
		if (theSGWindowPtr == nil)
			return;

		err = SGSetGWorld(data->theSGComp, GetWindowPort(theSGWindowPtr), nil); // GetWindowPort is a Carbon call but is in CarbonAccessors.o BJS
		//err = SGSetGWorld(data->theSGComp, nil, nil); // also works - BJS
		//err = SGSetGWorld(data->theSGComp, (CGrafPtr) theSGWindowPtr, nil); // this caused the app to quit in OS X BJS
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			CloseComponent(data->theSGComp);
			data->theSGComp = nil;
			return;
			}

	REALSetControlVisible(instance, 0L); // BJS

	captureSetEnableVideo(instance, 0, data->enableVideo);
	
	captureSetEnableSound(instance, 0, data->enableSound);
	}
}

static void captureDispose(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	//theSGComp
	//theSGVideoChannel, theSGSoundChannel;
	//close the channels
	
	/*
	if(data->theSGSoundChannel)
		SGDisposeChannel(data->theSGComp, data->theSGSoundChannel);
	if(data->theSGVideoChannel)
		SGDisposeChannel(data->theSGComp, data->theSGVideoChannel);

	CloseComponent(data->theSGComp);
	*/
}

#pragma mark captureDraw
#ifdef TARGET_WIN32
static void captureDraw(REALcontrolInstance instance, REALgraphics context)
#else
static void captureDraw(REALcontrolInstance instance)
#endif
{
	// Find the bounds rect
	Rect r;
	REALGetControlBounds(instance, &r);
	
	// Get our custom data into "data"
	ControlData(captureControl, instance, captureData, data);

	// Do the drawing (using native graphics toolcapture)
  #if TARGET_WIN32
	// Windows implementation - the SG should take care of all drawing, so the code that went here should not be needed - BJS

  #else
	// Mac implementation:

	RGBColor oldCol, fillCol, lineCol = {0,0,0};
	//PicHandle		IDEGraphicsHandle;
	Rect			myRect;
	short			myWidth,myHeight;
	Rect			pictRect;
	//short			pictWidth, pictHeight;
	//MatrixRecord	theMatrix;

if (REALGetControlVisible(instance) == 1L) // BJS
	{
	if (!REALinRuntime())
		{
		GetForeColor(&oldCol);
		fillCol.red = 0;
		fillCol.green = 0;
		fillCol.blue = 0;
		RGBForeColor(&fillCol);
		PaintRect(&r); // r is entire control
		
		fillCol.red = 32767;
		fillCol.green = 32767;
		fillCol.blue = 32767;
		RGBForeColor(&fillCol);
		FrameRect(&r);
	
		MoveTo(r.left, r.top);
		//LineTo(r.right - 1, r.bottom - 1);

		//********************************************
		//
		// draw the control's icon in the interface
		//
		//********************************************

	 		 	
	 	REALGetControlBounds(instance, &myRect);
	 		
	 	InsetRect(&myRect, 1, 1);
	 	myWidth = myRect.right - myRect.left;
	 	myHeight = myRect.bottom - myRect.top;
	 		
	 	if (myWidth > 82 && myHeight > 62)
	 		{
	 		pictRect.left = myRect.left + ((myWidth - 80) / 2);
	 		pictRect.top = myRect.top + ((myHeight - 60) / 2);
	 		pictRect.right = pictRect.left + 80;
	 		pictRect.bottom = pictRect.top + 60;
	 		
	 		
			// move through rects: left to right, top to bottom
			
			// top row
			// grey
			myRect.left = pictRect.left;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 52480;
			fillCol.green = 52480;
			fillCol.blue = 52480;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// yellow
			myRect.left = pictRect.left + 11;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 65535;
			fillCol.green = 65535;
			fillCol.blue = 0;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// blue/green
			myRect.left = pictRect.left + 23;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 0;
			fillCol.green = 65535;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// green
			myRect.left = pictRect.left + 34;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 0;
			fillCol.green = 65535;
			fillCol.blue = 0;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
		
			// red/blue
			myRect.left = pictRect.left + 46;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 65535;
			fillCol.green = 0;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
		
			// red
			myRect.left = pictRect.left + 57;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 65535;
			fillCol.green = 0;
			fillCol.blue = 0;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// blue
			myRect.left = pictRect.left + 69;
			myRect.top = pictRect.top;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 40;
			fillCol.red = 0;
			fillCol.green = 0;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// middle row
			// blue
			myRect.left = pictRect.left;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 0;
			fillCol.green = 0;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// really dark grey
			myRect.left = pictRect.left + 11;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// red/blue
			myRect.left = pictRect.left + 23;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 65535;
			fillCol.green = 0;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
			
			// really dark grey
			myRect.left = pictRect.left + 34;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
		
			// blue/green
			myRect.left = pictRect.left + 46;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 0;
			fillCol.green = 65535;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);
		
			// really dark grey
			myRect.left = pictRect.left + 57;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 12;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// grey
			myRect.left = pictRect.left + 69;
			myRect.top = pictRect.top + 40;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 5;
			fillCol.red = 52480;
			fillCol.green = 52480;
			fillCol.blue = 52480;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// bottom row
			// dark blue
			myRect.left = pictRect.left;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 14;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 2304;
			fillCol.green = 16128;
			fillCol.blue = 23040;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// white
			myRect.left = pictRect.left + 14;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 14;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 65535;
			fillCol.green = 65535;
			fillCol.blue = 65535;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// purple
			myRect.left = pictRect.left + 28;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 14;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 15104;
			fillCol.green = 0;
			fillCol.blue = 32512;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// really dark grey
			myRect.left = pictRect.left + 42;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 15;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// black
			myRect.left = pictRect.left + 57;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 4;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 0;
			fillCol.green = 0;
			fillCol.blue = 0;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// really dark grey
			myRect.left = pictRect.left + 61;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 4;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// dark grey
			myRect.left = pictRect.left + 65;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 4;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 9984;
			fillCol.green = 9984;
			fillCol.blue = 9984;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			// really dark grey
			myRect.left = pictRect.left + 69;
			myRect.top = pictRect.top + 45;
			myRect.right = myRect.left + 11;
			myRect.bottom = myRect.top + 15;
			fillCol.red = 5120;
			fillCol.green = 5120;
			fillCol.blue = 5120;
			RGBForeColor(&fillCol);
			PaintRect(&myRect);

			fillCol.red = 32767;
			fillCol.green = 32767;
			fillCol.blue = 32767;
			RGBForeColor(&fillCol);
			FrameRect(&pictRect);

			}
		
		RGBForeColor(&oldCol);

		}
	else // is in RunTime
		{
		if (!data->previewState || (data->recordState && !data->previewWhileRecording))
			{
	 		
	 		REALGetControlBounds(instance, &myRect);
	 		
	 		myWidth = myRect.right - myRect.left;
	 		myHeight = myRect.bottom - myRect.top;
	 		
	 		if (data->theNoPreviewPicture)
	 			{
	 			REALDrawPicturePrimitive(data->theNoPreviewPicture, &myRect, 0);
	 			}
	 		else
	 			{
				// draw blue if there's no REALpict set for no preview
				GetForeColor(&oldCol);
				fillCol.red = 0;//(data->fillColor >> 16) * 257;
				fillCol.green = 0; //((data->fillColor >> 8) & 0xff) * 257;
				fillCol.blue = 65535; //((data->fillColor & 0xff) * 257);
				RGBForeColor(&fillCol);
				PaintRect(&r);

				RGBForeColor(&lineCol);
				FrameRect(&r);
	
				MoveTo(r.left, r.top);
	 			}
			}
		else
			{
 			if (data->theSGComp)
 				{
 				if (data->theSGVideoChannel)
 					{
 					ComponentResult err;
 					SGPause(data->theSGComp, true);
 					SGGetSrcVideoBounds(data->theSGVideoChannel, &data->theNaturalBounds);
					REALGetControlBounds(instance, &data->theSGBounds);
					
					if (data->theSGBounds.right - data->theSGBounds.left > data->theNaturalBounds.right - data->theNaturalBounds.left)
						{
						//data->theSGBounds.left = data->theNaturalBounds.left;
						data->theSGBounds.right = data->theSGBounds.left + data->theNaturalBounds.right;
						}
					
					if (data->theSGBounds.bottom - data->theSGBounds.top > data->theNaturalBounds.bottom - data->theNaturalBounds.top)
						{
						//data->theSGBounds.top = data->theNaturalBounds.top;
						data->theSGBounds.bottom = data->theSGBounds.top + data->theNaturalBounds.bottom;
						}
					
					
					
 					/*
					err = SGGetChannelMatrix(data->theSGVideoChannel, &theMatrix);
					if (err != QT::noErr)
						{
						captureFireErrorEvent(instance, err);
						return;
						}
					else
						{
						RectMatrix(&theMatrix, &data->theNaturalBounds, &data->theSGBounds); // this stuff should either go or be put in it's own function
						//RotateMatrix(&theMatrix, 90, 0, 0);
						}
					*/
					//err = SGSetVideoRect(data->theSGVideoChannel, &data->theSGBounds); // test BJS 1/15/03
					//err = SGSetChannelMatrix(data->theSGVideoChannel, &theMatrix); //test BJS 1/15/03
					err = SGSetChannelBounds(data->theSGVideoChannel, &data->theSGBounds);
					if (err != QT::noErr)
						{
						captureFireErrorEvent(instance, err);
						return;
						}
					SGPause(data->theSGComp, data->pauseState);
					}
 				RgnHandle mySGRegion = NewRgn();
 				RectRgn(mySGRegion, &data->theSGBounds);
 				SGUpdate(data->theSGComp, mySGRegion);
				}
			}
		}
	}
	
   #endif
}

// BJS Added 4/25/03

static void captureSetFastGrabPicture(REALcontrolInstance instance, long param, Boolean value)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	data->fastGrabPicture = value;
}

static Boolean captureGetFastGrabPicture(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	return data->fastGrabPicture;
}

// end BJS addition 4/25/03

static REALpicture captureGrabPict(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	PicHandle thePicture;
	Rect theBounds;
	
	theBounds = data->theNaturalBounds;
	
	if (data->fastGrabPicture)
		{
		SGPause(data->theSGComp, true); // Only needed if grabPictCurrentImage flag is set BJS
		err = SGGrabPict(data->theSGComp, &thePicture, &theBounds, 0, grabPictIgnoreClip | grabPictCurrentImage);
		SGPause(data->theSGComp, data->pauseState); // Only needed if grabPictCurrentImage flag is set BJS
		}
	else
		err = SGGrabPict(data->theSGComp, &thePicture, &theBounds, 0, grabPictIgnoreClip);
	
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return nil;
		}
	return REALBuildPictureFromPicHandle(thePicture, true);
	
}

static void captureSetEnableVideo(REALcontrolInstance instance, long param, Boolean value)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	//SGChannel theChannel;
	//OSType theChannelType;
	
	data->enableVideo = value;
	
	if (REALinRuntime())
		{
		if (!data->theSGComp)
			return;
		
		if (data->theSGVideoChannel)
			{
			err = SGDisposeChannel(data->theSGComp, data->theSGVideoChannel);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					return;
					}
				else
					{
					REALSetControlVisible(instance, 0L); // BJS
					data->theSGVideoChannel = nil;
					}
			}
			
		if (data->enableVideo)
			{
			if (!data->theSGVideoChannel)
				err = SGNewChannel (data->theSGComp, VideoMediaType, &data->theSGVideoChannel);
				if (err != QT::noErr)
					{
					data->enableVideo = false;
					data->theSGVideoChannel = nil;
					REALSetControlVisible(instance, 0L); // BJS
					captureFireErrorEvent(instance, err);
					return;
					}
				else
					{
					err = SGGetSrcVideoBounds(data->theSGVideoChannel, &data->theNaturalBounds);
					if (err != QT::noErr)
						{
						captureFireErrorEvent(instance, err);
						}
						
					REALGetControlBounds(instance, &data->theSGBounds);
					
					err = SGSetChannelBounds(data->theSGVideoChannel, &data->theSGBounds);
					if (err != QT::noErr)
						{
						captureFireErrorEvent(instance, err);
						return;
						}
					else
						{
						captureSetPreviewState(instance, 0, data->previewState);
						captureSetPreviewWhileRecordingState(instance, 0, data->previewWhileRecording);
						}
					}
			REALSetControlVisible(instance, 1L); // BJS
			}
		}
	
		
}

static Boolean captureGetEnableVideo(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);

	return data->enableVideo;
	
}

static void captureSetEnableSound(REALcontrolInstance instance, long param, Boolean value)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	
	data->enableSound = value;
	

	if (REALinRuntime())
		{
		if (!data->theSGComp)
			return;
		
		if (data->theSGSoundChannel)
			{
			err = SGDisposeChannel(data->theSGComp, data->theSGSoundChannel);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return;
				}
			else
				data->theSGSoundChannel = nil;
			}
			
		if (data->enableSound)
			{
			if (!data->theSGSoundChannel)
				err = SGNewChannel (data->theSGComp, SoundMediaType, &data->theSGSoundChannel);
				if (err != QT::noErr)
					{
					data->enableSound = false;
					data->theSGSoundChannel = nil;
					captureFireErrorEvent(instance, err);
					return;
					}
				else
					{

					Handle	myRates = nil;
					
					myRates = NewHandleClear(5 * sizeof(Fixed));
					if (myRates != nil) 
						{
							*((long *)(*myRates) + 0) = Long2Fix(8000);  //8 kHz
							*((long *)(*myRates) + 1) = Long2Fix(11025); //11 kHz
							*((long *)(*myRates) + 2) = Long2Fix(16000); //16 kHz
							*((long *)(*myRates) + 3) = Long2Fix(22050); //22 kHz
							*((long *)(*myRates) + 4) = Long2Fix(32000); //32 kHz
							SGSetAdditionalSoundRates(data->theSGSoundChannel, myRates);
							
							DisposeHandle(myRates);
						}

					err = SGSetSoundInputParameters(data->theSGSoundChannel, data->sampleSize, data->numChan, 'raw ');
					if (err != QT::noErr)
						captureFireErrorEvent(instance, err);
				
					err = SGSetSoundInputRate(data->theSGSoundChannel, data->sampleRate);
					if (err != QT::noErr)
						captureFireErrorEvent(instance, err);

					captureSetPreviewState(instance, 0, data->previewState);
					captureSetPreviewWhileRecordingState(instance, 0, data->previewWhileRecording);
					}
			}

		}

}

static Boolean captureGetEnableSound(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	return data->enableSound;
}


// BJS added 12/26/02

static REALstring captureGetSettings(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	Handle theHandle;
	REALstring theUDString;
	UserData theUserData = 0L;
	
	
	if (REALinRuntime())
		{
		err = SGGetSettings(data->theSGComp, &theUserData, 0L);
		if (err != noErr)
		{
		captureFireErrorEvent(instance, err);
		return nil;
		}
		
		theHandle = NewHandle(0);
		
		if (!theHandle)
			return nil;
		
		err = PutUserDataIntoHandle(theUserData, theHandle);
		if (err != noErr)
		{
		captureFireErrorEvent(instance, err);
		return nil;
		}
		
		theUDString = REALBuildString((const char *)*theHandle, GetHandleSize(theHandle));
		
		DisposeHandle(theHandle);
		DisposeUserData(theUserData);
		return theUDString;
		}
	else
	return nil;
}

static void captureSetSettings(REALcontrolInstance instance, long param, REALstring theSettings)
	{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime)
		{
		ComponentResult err = QT::noErr;
		Handle theHandle;
		OSType theChannelType;
		SGChannel theChannel;
		short i;
		UserData theUserData = 0L;
		int theLen = 0;
	
		data->theSGVideoChannel = 0L;
		data->theSGSoundChannel = 0L;
		
		theLen = theSettings->Length();
		
		theHandle = NewHandle(theLen);
		
		BlockMove(REALCString(theSettings), *theHandle, theLen);
		
		err = NewUserDataFromHandle(theHandle, &theUserData);
		if (err != noErr)
			{
			captureFireErrorEvent(instance, err);
			return;
			}
		
		if (theUserData == 0L)
			{
			captureFireErrorEvent(instance, -666);
			return;
			}
		
		err = QT::noErr;
			
		err = SGSetSettings(data->theSGComp, theUserData, 0L);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			}
		
		DisposeUserData(theUserData);
		DisposeHandle(theHandle);

		err = QT::noErr;
		i = 1;
		
		while(err == QT::noErr)
			{
			err = SGGetIndChannel(data->theSGComp, i, &theChannel, &theChannelType);
			if (err == QT::noErr)
				{
				if (theChannel)
					{
					if (theChannelType == VideoMediaType)
						data->theSGVideoChannel = theChannel;
					else if (theChannelType == SoundMediaType)
						data->theSGSoundChannel = theChannel;
					}
				}
			i++;
			}
		}
	}

// end BJS addition 12/26/02

// BJS added 01/15/03

static short captureGetPreviewMode(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	long			thePlayFlags;
	
	
	if (REALinRuntime())
		{
		err = SGGetChannelPlayFlags(data->theSGVideoChannel, &thePlayFlags);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			}
		
		if (thePlayFlags == channelPlayNormal)
			return 0;
			
		if (thePlayFlags == channelPlayFast)
			return 1;
		
		if (thePlayFlags == channelPlayHighQuality)
			return 2;
		
		if (thePlayFlags == channelPlayAllData)
			return 3;
		}
	else
	{return 0;}
	return 0;
}

static void captureSetPreviewMode(REALcontrolInstance instance, long param, short theMode)
	{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	long	thePlayFlags;
	
	if (REALinRuntime)
		{
		// this is in case they set theMode to an invalid value
		thePlayFlags = channelPlayNormal;
		
		if (theMode == 0)
			thePlayFlags = channelPlayNormal;
		
		if (theMode == 1)
			thePlayFlags = channelPlayFast;
		
		if (theMode == 2)
			thePlayFlags = channelPlayHighQuality;
			
		if (theMode == 3)
			thePlayFlags = channelPlayAllData;
		
		err = SGSetChannelPlayFlags(data->theSGVideoChannel, thePlayFlags);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			}
		}
	}

// end BJS addition 01/15/03

static long captureGetVideoWidth(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			SGGetSrcVideoBounds(data->theSGVideoChannel, &data->theNaturalBounds);
			
			return data->theNaturalBounds.left + data->theNaturalBounds.right;
			}
		else
			return 0;
		}
	return 0;
}

static long captureGetVideoHeight(REALcontrolInstance instance, long param)
{
	#pragma unused( param )
	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			SGGetSrcVideoBounds(data->theSGVideoChannel, &data->theNaturalBounds);
			return data->theNaturalBounds.top + data->theNaturalBounds.bottom;
			}
		else
			return 0;
		}
	return 0;
}



static long captureGetSoundDeviceCurrent(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			err = SGGetChannelDeviceList(data->theSGSoundChannel, nil, &data->theSGSoundDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return 0;
				}
			return (**data->theSGSoundDeviceList).selectedIndex;
			}
		else
			return 0;
		}
	return 0;
}

static long captureGetSoundDeviceCount(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			err = SGGetChannelDeviceList(data->theSGSoundChannel, nil, &data->theSGSoundDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return 0;
				}
			return (**data->theSGSoundDeviceList).count;
			}
		else
			return 0;
		}
	return 0;
}

static REALstring captureGetIndexedSoundDeviceName(REALcontrolInstance instance, int index)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	short 				myCount;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			//************************
			//
			//    GET THE LIST
			//
			//************************
			err = SGGetChannelDeviceList(data->theSGSoundChannel, nil, &data->theSGSoundDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return nil;
				}
			//************************
			//
			//    GET THE LIST COUNT
			//
			//************************
			myCount = (**data->theSGSoundDeviceList).count;
			
			
			if (myCount)// != 0)
				{
				if (index > myCount-1 || index < 0)
					return REALBuildString("Invalid Index", 13);
			//************************
			//
			//    GET DEVICE NAME
			//
			//************************
				SGDeviceName 		tempDeviceName;
				tempDeviceName = (**data->theSGSoundDeviceList).entry[index];
				return REALstringFromPString(tempDeviceName.name);
				}
			}
		else
			return nil;
		}
	return nil;
}

static Boolean captureGetIndexedSoundDeviceAvailable(REALcontrolInstance instance, int index)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	short 				myCount;
	long				myFlags;
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			//************************
			//
			//    GET THE LIST
			//
			//************************
			err = SGGetChannelDeviceList(data->theSGSoundChannel, nil, &data->theSGSoundDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return false;
				}
			//************************
			//
			//    GET THE LIST COUNT
			//
			//************************
			myCount = (**data->theSGSoundDeviceList).count;
			
			
			if (myCount)// != 0)
				{
				if (index > myCount-1 || index < 0)
					return false;
			//************************
			//
			//    GET DEVICE NAME
			//
			//************************
				SGDeviceName 		tempDeviceName;
				tempDeviceName = (**data->theSGSoundDeviceList).entry[index];
				myFlags = tempDeviceName.flags;
				if (myFlags == sgDeviceNameFlagDeviceUnavailable)
					return false;
				else
					return true;
				}
			}
		else
			return false;
		}
	return false;
}

static void captureSetSoundDeviceName(REALcontrolInstance instance, REALstring theDeviceName)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	const unsigned char	* thePStringName;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			thePStringName = REALPString(theDeviceName);
			err = SGSetChannelDevice(data->theSGSoundChannel, (unsigned char *)thePStringName);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return;
				}
			}
		else
			return;
		}
	return;
}

static long captureGetCurrentSoundDeviceSourceCount(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err;
	Handle sourceNames;
	short numNames;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum, siInputSourceNames, &sourceNames);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					return 0;
					}
				numNames = (*(short**)sourceNames)[0];
				return numNames;
				}
			return 0;
			}
		return 0;
		}
return 0;
}

static REALstring captureGetIndexedSoundDeviceSource(REALcontrolInstance instance, int index)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err;
	Handle sourceNames;
	char sourceName[255];
	short numNames;
	long offset;
	int i;

	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum, siInputSourceNames, &sourceNames);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					return nil;
					}
				index++;
				
				numNames = (*(short**)sourceNames)[0];
				if (index < 0 || index > numNames)
					return REALBuildString("Invalid Index", 13);
				offset = 3;
				for (i = 0; i < index ; i++)
					{
					BlockMoveData (&((char*)(*sourceNames))[offset], sourceName, (*(char**)sourceNames)[offset-1]);
					sourceName[(*(char**)sourceNames)[offset-1]] = 0;
					
					offset += (*(char**)sourceNames)[offset-1] + 1;
					}

				return REALBuildString((const char *)&sourceName, strlen(sourceName));
				}
			return nil;
			}
		return nil;
		}
return nil;
}

static long captureGetCurrentSoundDeviceSource(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	short sourceIndex;
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum, siInputSource, &sourceIndex);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					return -1;
					}
				return sourceIndex - 1; // just to keep a consistant 0 based indexing scheme BJS
				}
			return -1;
			}
		return -1;
		}
	return -1;
}

static void captureSetSoundDeviceSourceIndex(REALcontrolInstance instance, short index)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				index++;
				
				err = SPBSetDeviceInfo(data->inDevNum, siInputSource, &index);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					}
				}
			}
		}
}

static Boolean captureGetAGCState(REALcontrolInstance instance, long param)
{

	#pragma unused( param )
	ComponentResult err;
	short agcState;

	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum,siAGCOnOff, (QT::Ptr) &agcState);
				if (err == QT::noErr)
					return agcState;
				else
					{
					captureFireErrorEvent(instance, err);
					return false;
					}
				return false;
				}
			return false;
			}
		return false;
		}
	return false;
}

static void captureSetAGCState(REALcontrolInstance instance, long param, Boolean value)
{

	#pragma unused( param )
	ComponentResult err;
	short agcState;

	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				agcState = value;
				err = SPBSetDeviceInfo(data->inDevNum,siAGCOnOff, (QT::Ptr) &agcState);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					}
				}
			}
		}	

}

static long captureGetGain(REALcontrolInstance instance, long param)
{

	#pragma unused( param )
	ComponentResult err;
	short theGain;

	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum,siInputGain, (QT::Ptr) &theGain);
				if (err == QT::noErr)
					return theGain;
				else
					{
					captureFireErrorEvent(instance, err);
					return 0;
					}
				return 0;
				}
			return 0;
			}
		return 0;
		}
	return 0;
}

static void captureSetGain(REALcontrolInstance instance, long param, long value)
{

	#pragma unused( param )
	ComponentResult err;
	short theGain;


	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				theGain = value;
				err = SPBSetDeviceInfo(data->inDevNum,siInputGain, (QT::Ptr) &theGain);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					}
				}
			}
		}	

}

static Boolean captureGetMeterState(REALcontrolInstance instance, long param)
{

	#pragma unused( param )
	ComponentResult err;

	struct soundLevelData
		{
		short meterState;
		short meterSetting;
		} levelMeterInfo;

	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				err = SPBGetDeviceInfo(data->inDevNum,siLevelMeterOnOff, (QT::Ptr) &levelMeterInfo);
				if (err == QT::noErr)
					return levelMeterInfo.meterState;
				else
					{
					captureFireErrorEvent(instance, err);
					return false;
					}
				return false;
				}
			return false;
			}
		return false;
		}
	return false;
}

static void captureSetMeterState(REALcontrolInstance instance, long param, Boolean value)
{

	#pragma unused( param )
	ComponentResult err;

	struct soundLevelData
		{
		short meterState;
		short meterSetting;
		} levelMeterInfo;


	ControlData(captureControl, instance, captureData, data);
	
	if (REALinRuntime())
		{
		if (data->theSGSoundChannel)
			{
			data->inDevNum = SGGetSoundInputDriver(data->theSGSoundChannel);
			if (data->inDevNum >= 0L)
				{
				levelMeterInfo.meterState = value;
				err = SPBSetDeviceInfo(data->inDevNum,siLevelMeterOnOff, (QT::Ptr) &levelMeterInfo);
				if (err != QT::noErr)
					{
					captureFireErrorEvent(instance, err);
					}
				}	
			}
		}
	
}

static short captureGetLeftLevel(REALcontrolInstance instance)
{
	ComponentResult err;

	struct LevelMeterInfo
	{
  		short               leftMeter;              // 0-255 range	
		short               rightMeter;             // 0-255 range
	} soundLevelInfo;

	ControlData(captureControl, instance, captureData, data);
	
	if (data->inDevNum)
		err = SPBGetDeviceInfo(data->inDevNum, siActiveLevels, (QT::Ptr) &soundLevelInfo);
	if (err == QT::noErr)
		return soundLevelInfo.leftMeter;
	else
		{
		captureFireErrorEvent(instance, err);
		return 0;
		}
	
	return 0;
}

static short captureGetRightLevel(REALcontrolInstance instance)
{
	ComponentResult err;

	struct LevelMeterInfo
	{
  		short               leftMeter;              // 0-255 range	
		short               rightMeter;             // 0-255 range
	} soundLevelInfo;

	ControlData(captureControl, instance, captureData, data);
	
	if (data->inDevNum)
		err = SPBGetDeviceInfo(data->inDevNum, siActiveLevels, (QT::Ptr) &soundLevelInfo);
	if (err == QT::noErr)
		return soundLevelInfo.rightMeter;
	else
		{
		captureFireErrorEvent(instance, err);
		return 0;
		}
	
	return 0;
}

static long captureGetVideoDeviceCurrent(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;

	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			err = SGGetChannelDeviceList(data->theSGVideoChannel, nil, &data->theSGVideoDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return 0;
				}
			return (**data->theSGVideoDeviceList).selectedIndex;
			}
		else
			return 0;
		}
	return 0;
}

static long captureGetVideoDeviceCount(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;

	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			err = SGGetChannelDeviceList(data->theSGVideoChannel, nil, &data->theSGVideoDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return 0;
				}
			return (**data->theSGVideoDeviceList).count;
			}
		else
			return 0;
		}
	return 0;
}

static REALstring captureGetIndexedVideoDeviceName(REALcontrolInstance instance, int index)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	short 				myCount;

	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			//************************
			//
			//    GET THE LIST
			//
			//************************
			err = SGGetChannelDeviceList(data->theSGVideoChannel, nil, &data->theSGVideoDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return nil;
				}
			//************************
			//
			//    GET THE LIST COUNT
			//
			//************************
			myCount = (**data->theSGVideoDeviceList).count;
			
			
			if (myCount)// != 0)
				{
				if (index > myCount-1 || index < 0)
					return REALBuildString("Invalid Index", 13);
			//************************
			//
			//    GET DEVICE NAME
			//
			//************************
				SGDeviceName 		tempDeviceName;
				tempDeviceName = (**data->theSGVideoDeviceList).entry[index];
				return REALstringFromPString(tempDeviceName.name);
				}
			}
		else
			return nil;
		}
	return nil;
}

static Boolean captureGetIndexedVideoDeviceAvailable(REALcontrolInstance instance, int index)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	short 				myCount;
	long				myFlags;
	
	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			//************************
			//
			//    GET THE LIST
			//
			//************************
			err = SGGetChannelDeviceList(data->theSGVideoChannel, nil, &data->theSGVideoDeviceList);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return false;
				}
			//************************
			//
			//    GET THE LIST COUNT
			//
			//************************
			myCount = (**data->theSGVideoDeviceList).count;
			
			
			if (myCount)// != 0)
				{
				if (index > myCount-1 || index < 0)
					return false;
			//************************
			//
			//    GET DEVICE NAME
			//
			//************************
				SGDeviceName 		tempDeviceName;
				tempDeviceName = (**data->theSGVideoDeviceList).entry[index];
				myFlags = tempDeviceName.flags;
				if (myFlags == sgDeviceNameFlagDeviceUnavailable)
					return false;
				else
					return true;
				}
			}
		else
			return false;
		}
	return false;
}

static void captureSetVideoDeviceName(REALcontrolInstance instance, REALstring theDeviceName)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult 	err;
	const unsigned char	* thePStringName;

	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			thePStringName = REALPString(theDeviceName);
			err = SGSetChannelDevice(data->theSGVideoChannel, (unsigned char *)thePStringName);
			if (err != QT::noErr)
				{
				captureFireErrorEvent(instance, err);
				return;
				}
			}
		else
			return;
		}
	return;
}

/*
static void captureSetVideoDeviceTimeCodeEnabled(REALcontrolInstance instance, long param, Boolean theState)
{
	ControlData(captureControl, instance, captureData, data);

	if (REALinRuntime())
		{
		if (data->theSGVideoChannel)
			{
			data->theVideoComponent = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
			if (data->theVideoComponent != nil)
				
			}
		}

//SGGetVideoDigitizerComponent
}
*/

static void captureSetVideoDeviceSourceIndex(REALcontrolInstance instance, long theDeviceIndex)
{
	#pragma unused(theDeviceIndex)
	ControlData(captureControl, instance, captureData, data);
	
}

REALstring REALstringFromPString(const unsigned char *pstring)
  {
  REALstring result = nil;
  if (pstring[0] != 0) { // empty strings don't get converted
    result = REALBuildString((const char *)pstring+1, (int)pstring[0]);
    }
  return result;
  }

//
static long captureGetSoundVolume(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ComponentResult err;

	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		err = SGGetChannelVolume(data->theSGSoundChannel, &data->soundVolume);
		if (err != QT::noErr)
			captureFireErrorEvent(instance, err);
	
	return data->soundVolume;
}

static void captureSetSoundVolume(REALcontrolInstance instance, long param, long theVolume)
{
	#pragma unused(param)
	ComponentResult err;

	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		{
		if (theVolume > 256)
			theVolume = 256;
		if (theVolume < 0)
			theVolume = 0;
			
		err = SGSetChannelVolume(data->theSGSoundChannel, theVolume);
		if (err != QT::noErr)
			captureFireErrorEvent(instance, err);
		else
			err = SGGetChannelVolume(data->theSGSoundChannel, &data->soundVolume);
		}
}
//

static long captureGetSampleRate(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		data->sampleRate = SGGetSoundInputRate(data->theSGSoundChannel);
	
	return UnsignedFixedMulDiv(data->sampleRate, 1, 65536);
}

static void captureSetSampleRate(REALcontrolInstance instance, long param, long theRate)
{
	#pragma unused(param)
	ComponentResult err;
	Fixed theSampleRate;

	ControlData(captureControl, instance, captureData, data);
	
	theSampleRate = UnsignedFixedMulDiv(theRate, 65536, 1);
	
	if (data->theSGSoundChannel)
		{
		err = SGSetSoundInputRate(data->theSGSoundChannel, theSampleRate);
		if (err != QT::noErr)
			captureFireErrorEvent(instance, err);
		else
			data->sampleRate = SGGetSoundInputRate(data->theSGSoundChannel);
		}
}

static long captureGetSampleSize(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	QT::OSErr err;
	
	
	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
	
	return data->sampleSize;
}

static void captureSetSampleSize(REALcontrolInstance instance, long param, long theSize)
{
	#pragma unused(param)
	QT::OSErr err;
	
	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		{
		err = SGSetSoundInputParameters(data->theSGSoundChannel, theSize, data->numChan, data->soundCType);
		if (err != QT::noErr)
			captureFireErrorEvent(instance, err);
		else
		err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
		}
}

static long captureGetNumChan(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	QT::OSErr err;
	
	
	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
	
	return data->numChan;
}

static void captureSetNumChan(REALcontrolInstance instance, long param, long theNumChan)
{
	#pragma unused(param)
	QT::OSErr err;
	
	ControlData(captureControl, instance, captureData, data);
	
	if (data->theSGSoundChannel)
		{
		err = SGSetSoundInputParameters(data->theSGSoundChannel, data->sampleSize, theNumChan, data->soundCType);
		if (err != QT::noErr)
			captureFireErrorEvent(instance, err);
		else
			err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
		}
}

static REALstring captureGetSoundCompressionType(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	QT::OSErr err;
	OSType compType;
	
	ControlData(captureControl, instance, captureData, data);
	
	err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
	
	compType = data->soundCType;
	
	#ifdef WIN32
		//swap the bytes
		compType = uhtonl(compType);
	#endif

	return	REALBuildString((QT::Ptr) &compType, 4);	
}

static void captureSetSoundCompressionType(REALcontrolInstance instance, long param, REALstring theCompType)
{
	#pragma unused(param)
	QT::OSErr err;
	OSType compType;
	
	ControlData(captureControl, instance, captureData, data);
	
	compType = REALstringToOSType(theCompType);
	
	err = SGSetSoundInputParameters(data->theSGSoundChannel, data->sampleSize, data->numChan, compType);
	
	err = SGGetSoundInputParameters(data->theSGSoundChannel, &data->sampleSize, &data->numChan, &data->soundCType);
}

static long captureGetDepth(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return nil;
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
	return data->theDepth;
	
}

static void captureSetDepth(REALcontrolInstance instance, long param, long theDepth)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return;

	err = SGSetVideoCompressor(data->theSGVideoChannel, theDepth, nil, data->theSpatialQuality, data->theTemporalQuality, data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
}

static long captureGetSpatialQuality(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return nil;
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
	return data->theSpatialQuality;
	
}

static void captureSetSpatialQuality(REALcontrolInstance instance, long param, long theSpatialQuality)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return;

	err = SGSetVideoCompressor(data->theSGVideoChannel, data->theDepth, nil, theSpatialQuality, data->theTemporalQuality, data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
}

static long captureGetTemporalQuality(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return nil;
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
	return data->theTemporalQuality;
	
}

static void captureSetTemporalQuality(REALcontrolInstance instance, long param, long theTemporalQuality)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return;

	err = SGSetVideoCompressor(data->theSGVideoChannel, data->theDepth, nil, data->theSpatialQuality, theTemporalQuality, data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
}

static long captureGetKeyFrameRate(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return nil;
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
	return data->theKeyFrameRate;
	
}

static void captureSetKeyFrameRate(REALcontrolInstance instance, long param, long theKeyFrameRate)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return;

	err = SGSetVideoCompressor(data->theSGVideoChannel, data->theDepth, nil, data->theSpatialQuality, data->theTemporalQuality, theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
		
	err = SGGetVideoCompressor(data->theSGVideoChannel, &data->theDepth, nil, &data->theSpatialQuality, &data->theTemporalQuality, &data->theKeyFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
}

static double captureGetFrameRate(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	
	if (!data->theSGVideoChannel)
		return nil;
		
	err = SGGetFrameRate(data->theSGVideoChannel, &data->frameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
	double dRate = data->frameRate / 65536.0;
	return dRate;
}

static void captureSetFrameRate(REALcontrolInstance instance, long param, double theRate)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err;
	Fixed theFrameRate;
	
	if (!data->theSGVideoChannel)
		return;
	
	theFrameRate = theRate * 65536;

	err = SGSetFrameRate(data->theSGVideoChannel, theFrameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
		
	err = SGGetFrameRate(data->theSGVideoChannel, &data->frameRate);
	if (err != QT::noErr)
		captureFireErrorEvent(instance, err);
	
}

static REALstring captureGetVideoCompressionType(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	QT::OSErr err;
	OSType compType;
	
	ControlData(captureControl, instance, captureData, data);
	
	err = SGGetVideoCompressorType(data->theSGVideoChannel, &data->videoCType);
	
	compType = data->videoCType;
	
	#ifdef WIN32
		//swap the bytes
		compType = uhtonl(compType);
	#endif

	return	REALBuildString((QT::Ptr) &compType, 4);	
}

static void captureSetVideoCompressionType(REALcontrolInstance instance, long param, REALstring theCompType)
{
	#pragma unused(param)
	QT::OSErr err;
	OSType compType;
	
	ControlData(captureControl, instance, captureData, data);
	
	compType = REALstringToOSType(theCompType);
	
	err = SGSetVideoCompressorType(data->theSGVideoChannel, compType);
	
	err = SGGetVideoCompressorType(data->theSGVideoChannel, &data->videoCType);
}

// BJS added 11/12/02
static long captureGetVideoBlackLevelValue(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theBlackLevelValue;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the blacklevel value
	err = VDGetBlackLevelValue(theVideoDevice, &theBlackLevelValue);
	if (err != noErr)
		return err;
			
	return theBlackLevelValue;
}

static void captureSetVideoBlackLevelValue(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theBlackLevelValue;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the blacklevel value
	theBlackLevelValue = theValue;
	
	err = VDSetBlackLevelValue(theVideoDevice, &theBlackLevelValue);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoBrightness(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theBrightness;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the brightness 
	err = VDGetBrightness(theVideoDevice, &theBrightness);
	if (err != noErr)
		return err;
			
	return theBrightness;
}

static void captureSetVideoBrightness(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theBrightness;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the brightness 
	theBrightness = theValue;
	
	err = VDSetBrightness(theVideoDevice, &theBrightness);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoContrast(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theContrast;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the contrast 
	err = VDGetContrast(theVideoDevice, &theContrast);
	if (err != noErr)
		return err;
			
	return theContrast;
}

static void captureSetVideoContrast(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theContrast;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the contrast 
	theContrast = theValue;
	
	err = VDSetContrast(theVideoDevice, &theContrast);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoHue(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theHue;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the Hue 
	err = VDGetHue(theVideoDevice, &theHue);
	if (err != noErr)
		return err;
			
	return theHue;
}

static void captureSetVideoHue(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theHue;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the Hue 
	theHue = theValue;
	
	err = VDSetHue(theVideoDevice, &theHue);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoSaturation(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theSaturation;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the Saturation 
	err = VDGetSaturation(theVideoDevice, &theSaturation);
	if (err != noErr)
		return err;
			
	return theSaturation;
}

static void captureSetVideoSaturation(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theSaturation;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the Saturation 
	theSaturation = theValue;
	
	err = VDSetSaturation(theVideoDevice, &theSaturation);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoSharpness(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theSharpness;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the Sharpness 
	err = VDGetSharpness(theVideoDevice, &theSharpness);
	if (err != noErr)
		return err;
			
	return theSharpness;
}

static void captureSetVideoSharpness(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theSharpness;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the Sharpness 
	theSharpness = theValue;
	
	err = VDSetSharpness(theVideoDevice, &theSharpness);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


static long captureGetVideoWhiteLevelValue(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theWhiteLevelValue;
	
	if (!data->theSGComp)
		return -666;
	
	if (!data->theSGVideoChannel)
		return -6666;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return -66666;
	// get the whitelevel value
	err = VDGetWhiteLevelValue(theVideoDevice, &theWhiteLevelValue);
	if (err != noErr)
		return err;
			
	return theWhiteLevelValue;
}

static void captureSetVideoWhiteLevelValue(REALcontrolInstance instance, long param, long theValue)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
		
	ComponentResult err = noErr;
	ComponentInstance theVideoDevice;
	unsigned short theWhiteLevelValue;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGVideoChannel)
		return;
	
	// get the vdig
	theVideoDevice = SGGetVideoDigitizerComponent(data->theSGVideoChannel);
	if (!theVideoDevice)
		return;
	// set the whitelevel value
	theWhiteLevelValue = theValue;
	
	err = VDSetWhiteLevelValue(theVideoDevice, &theWhiteLevelValue);
	if (err != noErr)
		{captureFireErrorEvent(instance, err);
		return;
		}
}


// End BJS additions 11/12/02


//BC v2 additions - related to Windows API
//**************************************************************************************
//
//     MIN AND MAX GETTERS FOR VIDEO PARAMS
//
//**************************************************************************************
#pragma mark -

static long captureGetVideoBrightnessMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoBrightnessMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 

static long captureGetVideoContrastMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoContrastMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 

static long captureGetVideoHueMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoHueMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 

static long captureGetVideoSaturationMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoSaturationMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 

static long captureGetVideoSharpnessMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoSharpnessMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 

static long captureGetVideoWhiteLevelMax(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (16383);
} 

static long captureGetVideoWhiteLevelMin(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	return (0);
} 


#pragma mark -

/* ---------------------------------------------------------------------- */

static pascal Boolean
SeqGrabberModalFilterProc (DialogPtr theDialog, const EventRecord *theEvent,
	short *itemHit, long refCon)
{
#pragma unused(theDialog, itemHit)
	// Ordinarily, if we had multiple windows we cared about, we'd handle
	// updating them in here, but since we don't, we'll just clear out
	// any update events meant for us
	
	Boolean	handled = false;
	
	if ((theEvent->what == updateEvt) && 
		((WindowPtr) theEvent->message == (WindowPtr) refCon))
	{
		BeginUpdate ((WindowPtr) refCon);
		EndUpdate ((WindowPtr) refCon);
		handled = true;
	}
	return (handled);
}

/* ---------------------------------------------------------------------- */

//return( SGSettingsDialog([mSG seqGrabComponent], mChan, 0, NULL, 0, NULL, 0) );
static Boolean captureShowVideoSettingsDialog(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	Rect	newActiveVideoRect;
	Rect	adjustedActiveVideoRect;
	Rect	curBounds, curVideoRect, newVideoRect, newBounds;
	short	width, height;
	GrafPtr	savedPort;
	RgnHandle	deadRgn;
	SGModalFilterUPP	seqGragModalFilterUPP;
	Rect	portRect;	
	
	ComponentResult err = QT::noErr;
	
	if (!data->theSGComp)
		return false;
	
	if (!data->theSGVideoChannel)
		return false;
	

	// Get our current state
	err = SGGetChannelBounds (data->theSGVideoChannel, &curBounds);
	err = SGGetVideoRect (data->theSGVideoChannel, &curVideoRect);
	
	
	// Pause
	err = SGPause (data->theSGComp, true);
	
	// Do the dialog thang
	seqGragModalFilterUPP = (SGModalFilterUPP)NewSGModalFilterUPP(SeqGrabberModalFilterProc);
	//err = SGSettingsDialog(gSeqGrabber, gVideoChannel, 0, NULL, 0L, seqGragModalFilterUPP, (long)gMonitor);
	err = SGSettingsDialog(data->theSGComp, data->theSGVideoChannel, 0, NULL, 0L, seqGragModalFilterUPP, nil);
	DisposeSGModalFilterUPP(seqGragModalFilterUPP);
	
	
	//err = SGSettingsDialog (data->theSGComp, data->theSGVideoChannel, 0, nil, 0L, nil, nil);
	if (err != QT::noErr)
		return false;
	
	return true;
}

static Boolean captureShowSoundSettingsDialog(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);
	
	
	ConstComponentListPtr	myPanelList=0;
	ComponentResult err = QT::noErr;
	
	if (!data->theSGComp)
		return false;
	
	if (!data->theSGSoundChannel)
		return false;
		
	err = SGSettingsDialog (data->theSGComp, data->theSGSoundChannel, 1, myPanelList, 0L, nil, nil);
	if (err != QT::noErr)
		return false;
			
	return true;
}

static Boolean captureIsRecording(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	Boolean currentPreviewState;
	
	err = SGGetMode(data->theSGComp, &currentPreviewState, &data->recordState);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return data->recordState;
		}

	return data->recordState;

}

static void captureIdle(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	while (!Button() && (err == QT::noErr))
	{
		err = SGIdle(data->theSGComp);
		if (err != QT::noErr)
			err = SGStop(data->theSGComp);
			
		
	}
}

static void captureStartRecordingAsync(REALcontrolInstance instance, REALfolderItem theMovieFile)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	FSSpec theMovieSpec;
	
	REALFSSpecFromFolderItem(&theMovieSpec, theMovieFile);
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
		
	DeleteMovieFile(&theMovieSpec);
		
	err = CreateMovieFile(&theMovieSpec, 'TVOD', smSystemScript, createMovieFileDontOpenFile | createMovieFileDontCreateMovie | createMovieFileDontCreateResFile, nil, nil);
	if (err != QT::noErr)
		return;
	
	err = SGSetDataOutput(data->theSGComp, &theMovieSpec, seqGrabToDisk ); // | seqGrabAppendToFile
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	if (!data->previewWhileRecording)
		REALInvalidateControl(instance);
	
	err = SGSetMaximumRecordTime(data->theSGComp, 0);
						
	err = SGPrepare(data->theSGComp, true, true);
	
	captureFireRecordEvent(instance);
	
	err = SGStartRecord(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	err = SGIdle(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	data->recordState = true;
	data->pauseState = false;

}


static void captureStartRecordingAsyncTime(REALcontrolInstance instance, REALfolderItem theMovieFile, long theTicks)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	FSSpec theMovieSpec;
	
	REALFSSpecFromFolderItem(&theMovieSpec, theMovieFile);
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
		
	DeleteMovieFile(&theMovieSpec);
		
	err = CreateMovieFile(&theMovieSpec, 'TVOD', smSystemScript, createMovieFileDontOpenFile | createMovieFileDontCreateMovie | createMovieFileDontCreateResFile, nil, nil);
	if (err != QT::noErr)
		return;
	
	err = SGSetDataOutput(data->theSGComp, &theMovieSpec, seqGrabToDisk );
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	if (!data->previewWhileRecording)
		REALInvalidateControl(instance);
	
	err = SGSetMaximumRecordTime(data->theSGComp, theTicks);
						
	err = SGPrepare(data->theSGComp, true, true);
	
	captureFireRecordEvent(instance);

	err = SGStartRecord(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	err = SGIdle(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	data->recordState = true;
	data->pauseState = false;

}

static void captureStartRecordingSync(REALcontrolInstance instance, REALfolderItem theMovieFile)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	FSSpec theMovieSpec;
	
	REALFSSpecFromFolderItem(&theMovieSpec, theMovieFile);
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
		
	DeleteMovieFile(&theMovieSpec);
		
	err = CreateMovieFile(&theMovieSpec, 'TVOD', smSystemScript, createMovieFileDontOpenFile | createMovieFileDontCreateMovie | createMovieFileDontCreateResFile, nil, nil);
	if (err != QT::noErr)
		return;
	
	err = SGSetDataOutput(data->theSGComp, &theMovieSpec, seqGrabToDisk );
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	if (!data->previewWhileRecording)
		REALInvalidateControl(instance);
	
	data->recordState = true;

	err = SGSetMaximumRecordTime(data->theSGComp, 0);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
						
	err = SGPrepare(data->theSGComp, true, true);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		}
	
	captureFireRecordEvent(instance);

	err = SGStartRecord(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	err = SGIdle(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	FlushEvents(mDownMask+mUpMask+keyDownMask+keyUpMask+autoKeyMask,0);

	while (!Button() && err == QT::noErr)
		{	
		FlushEvents(mDownMask+mUpMask+keyDownMask+keyUpMask+autoKeyMask,0);
		err = SGIdle(data->theSGComp);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			return;
			}
		}

	err = SGStop(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	data->recordState = false;
	data->pauseState = false;

	err = SGRelease(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	if (data->previewState)
		captureStartPreviewing(instance);
	else
		captureDraw(instance);
		//REALInvalidateControl(instance);
}


static void captureStartRecordingSyncTime(REALcontrolInstance instance, REALfolderItem theMovieFile, long theTicks)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	FSSpec theMovieSpec;
	Boolean currentPreviewState;
	
	REALFSSpecFromFolderItem(&theMovieSpec, theMovieFile);
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
		
	DeleteMovieFile(&theMovieSpec);
		
	err = CreateMovieFile(&theMovieSpec, 'TVOD', smSystemScript, createMovieFileDontOpenFile | createMovieFileDontCreateMovie | createMovieFileDontCreateResFile, nil, nil);
	if (err != QT::noErr)
		return;
	
	err = SGSetDataOutput(data->theSGComp, &theMovieSpec, seqGrabToDisk );
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	if (!data->previewWhileRecording)
		REALInvalidateControl(instance);
	
	err = SGSetMaximumRecordTime(data->theSGComp, theTicks);
						
	err = SGPrepare(data->theSGComp, true, true);
	
	captureFireRecordEvent(instance);

	err = SGStartRecord(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
		
	err = SGGetMode(data->theSGComp, &currentPreviewState, &data->recordState);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}

	while (data->recordState && err == QT::noErr)
		{	
		FlushEvents(mDownMask+mUpMask+keyDownMask+keyUpMask+autoKeyMask,0);
		err = SGIdle(data->theSGComp);
		if (err != QT::noErr)
			{
			if (err != grabTimeComplete)
				captureFireErrorEvent(instance, err);
			return;
			}
		err = SGGetMode(data->theSGComp, &currentPreviewState, &data->recordState);
		if (err != QT::noErr)
			{
			if (err != grabTimeComplete)
				captureFireErrorEvent(instance, err);
			return;
			}
		}
/* The event filter handler calls captureStop, so we don't need to do this.
	err = SGStop(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	data->recordState = false;
	data->pauseState = false;

	err = SGRelease(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	if (data->previewState)
		captureStartPreviewing(instance);
	else
		captureDraw(instance);
		//REALInvalidateControl(instance);
*/
}







static void captureStartPreviewing(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
	else
		{
		err = SGStartPreview(data->theSGComp);
		if (err != QT::noErr)
			{
			data->previewState = false;
			data->recordState = false;
			data->pauseState = false;
			captureFireErrorEvent(instance, err);
			return;
			}
		else
		{
		data->previewState = true;
		data->recordState = false;
		data->pauseState = false;
		}
		}
}


static void captureSetPreviewState(REALcontrolInstance instance, long param, Boolean theState)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	long prevEnableFlag, prevWhileRecFlag;
		
	data->previewState = theState;
	
	if (REALinRuntime)
		{
		if (!data->theSGComp)
			return;
		else
			{	
			SGPause(data->theSGComp, true);

			prevEnableFlag = (data->previewState) ? seqGrabPreview : 0;
			prevWhileRecFlag = (data->previewWhileRecording) ? seqGrabPlayDuringRecord : 0;
	
			if (data->theSGSoundChannel)
				{
				err = SGSetChannelUsage (data->theSGSoundChannel, prevEnableFlag | seqGrabRecord | prevWhileRecFlag);
				if (err != QT::noErr)
					captureFireErrorEvent(instance, err);
				}
			
			if (data->theSGVideoChannel)
				{
				err = SGSetChannelUsage (data->theSGVideoChannel, prevEnableFlag | seqGrabRecord | prevWhileRecFlag);
				if (err != QT::noErr)
					captureFireErrorEvent(instance, err);
				}
		
			if (data->previewState)
				captureStartPreviewing(instance);
			else
				{
				if (!data->recordState)
					captureStop(instance);
					REALInvalidateControl(instance);
				}
			SGPause(data->theSGComp, data->pauseState);
			}
		}
}

static Boolean captureGetPreviewState(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	if (REALinRuntime())
		{
		err = SGGetMode(data->theSGComp, &data->previewState, &data->recordState);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			return data->previewState;
			}
		}
	
	return data->previewState;
}

static void captureSetPreviewWhileRecordingState(REALcontrolInstance instance, long param, Boolean theState)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	long prevEnableFlag, prevWhileRecFlag;
	
	data->previewWhileRecording = theState;

	if (REALinRuntime)
	{
	if (!data->theSGComp)
		return;
	
	SGPause(data->theSGComp, true);
	
	prevEnableFlag = (data->previewState) ? seqGrabPreview : 0;
	prevWhileRecFlag = (data->previewWhileRecording) ? seqGrabPlayDuringRecord : 0;
	
	if (data->theSGVideoChannel)
		err = SGSetChannelUsage (data->theSGVideoChannel, prevEnableFlag | seqGrabRecord | prevWhileRecFlag);
	
	if (data->theSGSoundChannel)
		err = SGSetChannelUsage (data->theSGSoundChannel, prevEnableFlag | seqGrabRecord | prevWhileRecFlag);

	SGPause(data->theSGComp, data->pauseState);
	}	
}

static Boolean captureGetPreviewWhileRecordingState(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);

	return data->previewWhileRecording;
}

static void captureSetPause(REALcontrolInstance instance, long param, Boolean theState)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	if (!data->theSGComp)
		return;
	
	if (!data->theSGSoundChannel && !data->theSGVideoChannel)
		return;
	
	data->pauseState = theState;

	if (theState)
		{
		err = SGPause(data->theSGComp, seqGrabPause);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			return;
			}
		}
	else
		{
		err = SGPause(data->theSGComp, seqGrabUnpause);
		if (err != QT::noErr)
			{
			captureFireErrorEvent(instance, err);
			return;
			}
		
		}
}

static Boolean captureGetPause(REALcontrolInstance instance, long param)
{
	#pragma unused(param)
	ControlData(captureControl, instance, captureData, data);
	
	Byte paused;
	ComponentResult err;
	
	err = SGGetPause(data->theSGComp, &paused);
	if (err == QT::noErr)
		{
		if (paused == seqGrabPause)
			data->pauseState = true;
		else
			data->pauseState = false;
		}
	
	return data->pauseState;
}

static void captureStop(REALcontrolInstance instance)
{
	ControlData(captureControl, instance, captureData, data);

	ComponentResult err = QT::noErr;
	
	err = SGStop(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	err = SGRelease(data->theSGComp);
	if (err != QT::noErr)
		{
		captureFireErrorEvent(instance, err);
		return;
		}
	
	if (data->previewState)
		captureStartPreviewing(instance);
	else
		captureDraw(instance);
		//REALInvalidateControl(instance);
	
	captureFireStopEvent(instance);
	
	data->recordState = false;
	data->pauseState = false;
}

static void captureFireErrorEvent(REALcontrolInstance instance, long theError)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);


	// invoke the event handler
	void (*fp)(REALcontrolInstance instance, int);
	fp = (void (*)(REALcontrolInstance instance, int)) REALGetEventInstance(instance, &captureEvents[0]);
	if (fp) fp(instance, theError);
}

static void captureFireRecordEvent(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);


	// invoke the event handler
	void (*fp)(REALcontrolInstance instance);
	fp = (void (*)(REALcontrolInstance instance)) REALGetEventInstance(instance, &captureEvents[1]);
	if (fp) fp(instance);
}

static void captureFireStopEvent(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);


	// invoke the event handler
	void (*fp)(REALcontrolInstance instance);
	fp = (void (*)(REALcontrolInstance instance)) REALGetEventInstance(instance, &captureEvents[2]);
	if (fp) fp(instance);
}

static void captureDeviceControlPoll(REALcontrolInstance instance)
{
	// get our private data
	ComponentResult err = QT::noErr;
	
	ControlData(captureControl, instance, captureData, data);

	err = SGIdle(data->theSGComp);


	
}


static void captureEventFilterHandler (EventRecord *event, long theInstance)
{

	#pragma unused(event)
	ComponentResult err = QT::noErr;
	Boolean currentPreviewState;
	
	if (theInstance != nil)
		{
		ControlData(captureControl, (REALcontrolInstance) theInstance, captureData, data);
	
		if (REALinRuntime)
			{
			if (data->theSGComp != nil)
				{
				err = SGIdle(data->theSGComp);
				err = SGGetMode(data->theSGComp, &currentPreviewState, &data->recordState);
				
				if (data->recordState || currentPreviewState)
					{
					err = SGIdle(data->theSGComp);
					if (err == grabTimeComplete)
						captureStop((REALcontrolInstance) theInstance);
					
					}		
				}
			}
		}

}


static void captureDeviceControlPlay(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	ComponentInstance dvInstance;
	
	ComponentDescription desc;
	desc.componentType = kIDHComponentType;
	desc.componentSubType = kIDHSubtypeDV;
	desc.componentManufacturer = NULL;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component dvComponent = FindNextComponent(NULL, &desc);

	if (!dvComponent)
		return;
	
	err = OpenAComponent(dvComponent, &dvInstance);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	QTAtomContainer deviceList;
	long numberDVDevices;
	
	err = IDHGetDeviceList(dvInstance, &deviceList);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	numberDVDevices = QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if(numberDVDevices == 0)
		{
		CloseComponent(dvInstance);
		return;
		}
	// debug only BJS
	captureFireErrorEvent(instance, numberDVDevices);
	
	for(long i=1; i <= numberDVDevices; ++i)
		{
		QTAtom deviceAtom; // get the atom to each device
		if (NULL == (deviceAtom = QTFindChildByIndex(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, i, NULL))){ continue; }
		
		QTAtom isochAtom; // find the isoch characteristics for this device
		if (NULL == (isochAtom = QTFindChildByIndex(deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, NULL))){ continue; }
		
		long nConfigs = QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
		
		captureFireErrorEvent(instance, nConfigs);
		
		for(long j = 1; j <= nConfigs; ++j)
			{						
			QTAtom configAtom; // get this configs atom
			if (NULL == (configAtom = QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, j, NULL))){ continue; }
			


		// BJS added
		char theName[256]; // get the name

		QTAtom nameAtom; 
		if (NULL == (nameAtom = QTFindChildByIndex(deviceList, isochAtom, kIDHNameAtomType, 1, NULL)))
			{
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size theSize;
			QTCopyAtomDataToPtr(deviceList, nameAtom, true, 255, &theName, &theSize);
			QTUnlockContainer(deviceList);
			//DebugStr(theName);

			REALMessageBox(REALBuildString((const char*) theName, theName[0]));
			}
		else
			SysBeep(30);



			QTAtom mediaAtom; // find the media type atom
			if (NULL == (mediaAtom = QTFindChildByIndex(deviceList, configAtom, kIDHIsochMediaType, 1, NULL))){ continue; }
			
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size siz;
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof mediaType, &mediaType, &siz);
			QTUnlockContainer(deviceList);
			
			if(kIDHVideoMediaAtomType == mediaType)
				{
				
				
				
				
				
				
				// found first appropriate media type.
				QTAtomSpec videoConfig;
				// found video device
				videoConfig.container = deviceList;
				videoConfig.atom = configAtom;
				
				err = IDHSetDeviceConfiguration(dvInstance, &videoConfig);
				if (err != QT::noErr)
					continue;
				
				ComponentInstance deviceControlInstance;
				
				err = IDHGetDeviceControl(dvInstance, &deviceControlInstance);
				if (err != QT::noErr)
					continue;

				
				char inBuffer[4];
				char outBuffer[16];
				DVCTransactionParams pParams;
				
				// fill up the avc frame
				inBuffer[0] = 0x00; //kAVCControlCommand;
				inBuffer[1] = 0x20;
				inBuffer[2] = 0xC3; // Play command is 0xc3 0x75
				inBuffer[3] = 0x75;
				
				// fill up the transaction parameter block
				pParams.commandBufferPtr = inBuffer;
				pParams.commandLength = sizeof inBuffer;
				pParams.responseBufferPtr = outBuffer;
				pParams.responseBufferSize = sizeof outBuffer;
				pParams.responseHandler = NULL;
				std::memset(outBuffer, 0, sizeof outBuffer);
				
				err = DeviceControlDoAVCTransaction(deviceControlInstance, &pParams);
				if (err != QT::noErr)
					{
					CloseComponent(dvInstance);
					captureFireErrorEvent(instance, err);
					return;
					}
					
				
				CloseComponent(dvInstance);
				return;
				
				
				
				
				
				
				}
				
			}//for2
		}//for1
 // nothing acceptable found.
 CloseComponent(dvInstance);
 return;
}

static void captureDeviceControlStop(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	ComponentInstance dvInstance;
	
	ComponentDescription desc;
	desc.componentType = kIDHComponentType;
	desc.componentSubType = kIDHSubtypeDV;
	desc.componentManufacturer = NULL;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component dvComponent = FindNextComponent(NULL, &desc);

	if (!dvComponent)
		return;
	
	err = OpenAComponent(dvComponent, &dvInstance);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	QTAtomContainer deviceList;
	long numberDVDevices;
	
	err = IDHGetDeviceList(dvInstance, &deviceList);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	numberDVDevices = QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if(numberDVDevices == 0)
		{
		CloseComponent(dvInstance);
		return;
		}

	for(long i=1; i <= numberDVDevices; ++i)
		{
		QTAtom deviceAtom; // get the atom to each device
		if (NULL == (deviceAtom = QTFindChildByIndex(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, i, NULL))){ continue; }
		
		QTAtom isochAtom; // find the isoch characteristics for this device
		if (NULL == (isochAtom = QTFindChildByIndex(deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, NULL))){ continue; }
		
		long nConfigs = QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
		
		for(long j = 1; j <= nConfigs; ++j)
			{
			QTAtom configAtom; // get this configs atom
			if (NULL == (configAtom = QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, j, NULL))){ continue; }
			
			QTAtom mediaAtom; // find the media type atom
			if (NULL == (mediaAtom = QTFindChildByIndex(deviceList, configAtom, kIDHIsochMediaType, 1, NULL))){ continue; }
			
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size siz;
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof mediaType, &mediaType, &siz);
			QTUnlockContainer(deviceList);
			
			if(kIDHVideoMediaAtomType == mediaType)
				{
				// found first appropriate media type.
				QTAtomSpec videoConfig;
				// found video device
				videoConfig.container = deviceList;
				videoConfig.atom = configAtom;
				
				err = IDHSetDeviceConfiguration(dvInstance, &videoConfig);
				if (err != QT::noErr)
					continue;
				
				ComponentInstance deviceControlInstance;
				
				err = IDHGetDeviceControl(dvInstance, &deviceControlInstance);
				if (err != QT::noErr)
					continue;

				
				char inBuffer[4];
				char outBuffer[16];
				DVCTransactionParams pParams;
				
				// fill up the avc frame
				inBuffer[0] = 0x00; //kAVCControlCommand;
				inBuffer[1] = 0x20;
				inBuffer[2] = 0xC4; // Stop command is 0xc4 0x60
				inBuffer[3] = 0x60;
				
				// fill up the transaction parameter block
				pParams.commandBufferPtr = inBuffer;
				pParams.commandLength = sizeof inBuffer;
				pParams.responseBufferPtr = outBuffer;
				pParams.responseBufferSize = sizeof outBuffer;
				pParams.responseHandler = NULL;
				std::memset(outBuffer, 0, sizeof outBuffer);
				
				err = DeviceControlDoAVCTransaction(deviceControlInstance, &pParams);
				if (err != QT::noErr)
					{
					CloseComponent(dvInstance);
					captureFireErrorEvent(instance, err);
					return;
					}
					
				
				CloseComponent(dvInstance);
				return;
				}
			}
		}
 // nothing acceptable found.
 CloseComponent(dvInstance);
 return;
}

static void captureDeviceControlFreeze(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	ComponentInstance dvInstance;
	
	ComponentDescription desc;
	desc.componentType = kIDHComponentType;
	desc.componentSubType = kIDHSubtypeDV;
	desc.componentManufacturer = NULL;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component dvComponent = FindNextComponent(NULL, &desc);

	if (!dvComponent)
		return;
	
	err = OpenAComponent(dvComponent, &dvInstance);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	QTAtomContainer deviceList;
	long numberDVDevices;
	
	err = IDHGetDeviceList(dvInstance, &deviceList);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	numberDVDevices = QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if(numberDVDevices == 0)
		{
		CloseComponent(dvInstance);
		return;
		}

	for(long i=1; i <= numberDVDevices; ++i)
		{
		QTAtom deviceAtom; // get the atom to each device
		if (NULL == (deviceAtom = QTFindChildByIndex(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, i, NULL))){ continue; }
		
		QTAtom isochAtom; // find the isoch characteristics for this device
		if (NULL == (isochAtom = QTFindChildByIndex(deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, NULL))){ continue; }
		
		long nConfigs = QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
		
		for(long j = 1; j <= nConfigs; ++j)
			{
			QTAtom configAtom; // get this configs atom
			if (NULL == (configAtom = QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, j, NULL))){ continue; }
			
			QTAtom mediaAtom; // find the media type atom
			if (NULL == (mediaAtom = QTFindChildByIndex(deviceList, configAtom, kIDHIsochMediaType, 1, NULL))){ continue; }
			
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size siz;
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof mediaType, &mediaType, &siz);
			QTUnlockContainer(deviceList);
			
			if(kIDHVideoMediaAtomType == mediaType)
				{
				// found first appropriate media type.
				QTAtomSpec videoConfig;
				// found video device
				videoConfig.container = deviceList;
				videoConfig.atom = configAtom;
				
				err = IDHSetDeviceConfiguration(dvInstance, &videoConfig);
				if (err != QT::noErr)
					continue;
				
				ComponentInstance deviceControlInstance;
				
				err = IDHGetDeviceControl(dvInstance, &deviceControlInstance);
				if (err != QT::noErr)
					continue;

				
				char inBuffer[4];
				char outBuffer[16];
				DVCTransactionParams pParams;
				
				// fill up the avc frame
				inBuffer[0] = 0x00; //kAVCControlCommand;
				inBuffer[1] = 0x20;
				inBuffer[2] = 0xC3; // Pause command is 0xc3 0x7d
				inBuffer[3] = 0x7D;
				
				// fill up the transaction parameter block
				pParams.commandBufferPtr = inBuffer;
				pParams.commandLength = sizeof inBuffer;
				pParams.responseBufferPtr = outBuffer;
				pParams.responseBufferSize = sizeof outBuffer;
				pParams.responseHandler = NULL;
				std::memset(outBuffer, 0, sizeof outBuffer);
				
				err = DeviceControlDoAVCTransaction(deviceControlInstance, &pParams);
				if (err != QT::noErr)
					{
					CloseComponent(dvInstance);
					captureFireErrorEvent(instance, err);
					return;
					}
					
				
				CloseComponent(dvInstance);
				return;
				}
			}
		}
 // nothing acceptable found.
 CloseComponent(dvInstance);
 return;
}

static void captureDeviceControlFastForward(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	ComponentInstance dvInstance;
	
	ComponentDescription desc;
	desc.componentType = kIDHComponentType;
	desc.componentSubType = kIDHSubtypeDV;
	desc.componentManufacturer = NULL;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component dvComponent = FindNextComponent(NULL, &desc);

	if (!dvComponent)
		return;
	
	err = OpenAComponent(dvComponent, &dvInstance);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	QTAtomContainer deviceList;
	long numberDVDevices;
	
	err = IDHGetDeviceList(dvInstance, &deviceList);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	numberDVDevices = QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if(numberDVDevices == 0)
		{
		CloseComponent(dvInstance);
		return;
		}

	for(long i=1; i <= numberDVDevices; ++i)
		{
		QTAtom deviceAtom; // get the atom to each device
		if (NULL == (deviceAtom = QTFindChildByIndex(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, i, NULL))){ continue; }
		
		QTAtom isochAtom; // find the isoch characteristics for this device
		if (NULL == (isochAtom = QTFindChildByIndex(deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, NULL))){ continue; }
		
		long nConfigs = QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
		
		for(long j = 1; j <= nConfigs; ++j)
			{
			QTAtom configAtom; // get this configs atom
			if (NULL == (configAtom = QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, j, NULL))){ continue; }
			
			QTAtom mediaAtom; // find the media type atom
			if (NULL == (mediaAtom = QTFindChildByIndex(deviceList, configAtom, kIDHIsochMediaType, 1, NULL))){ continue; }
			
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size siz;
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof mediaType, &mediaType, &siz);
			QTUnlockContainer(deviceList);
			
			if(kIDHVideoMediaAtomType == mediaType)
				{
				// found first appropriate media type.
				QTAtomSpec videoConfig;
				// found video device
				videoConfig.container = deviceList;
				videoConfig.atom = configAtom;
				
				err = IDHSetDeviceConfiguration(dvInstance, &videoConfig);
				if (err != QT::noErr)
					continue;
				
				ComponentInstance deviceControlInstance;
				
				err = IDHGetDeviceControl(dvInstance, &deviceControlInstance);
				if (err != QT::noErr)
					continue;

				
				char inBuffer[4];
				char outBuffer[16];
				DVCTransactionParams pParams;
				
				// fill up the avc frame
				inBuffer[0] = 0x00; //kAVCControlCommand;
				inBuffer[1] = 0x20;
				inBuffer[2] = 0xC4; // FFWD command is 0xc4 0x75
				inBuffer[3] = 0x75;
				
				// fill up the transaction parameter block
				pParams.commandBufferPtr = inBuffer;
				pParams.commandLength = sizeof inBuffer;
				pParams.responseBufferPtr = outBuffer;
				pParams.responseBufferSize = sizeof outBuffer;
				pParams.responseHandler = NULL;
				std::memset(outBuffer, 0, sizeof outBuffer);
				
				err = DeviceControlDoAVCTransaction(deviceControlInstance, &pParams);
				if (err != QT::noErr)
					{
					CloseComponent(dvInstance);
					captureFireErrorEvent(instance, err);
					return;
					}
					
				
				CloseComponent(dvInstance);
				return;
				}
			}
		}
 // nothing acceptable found.
 CloseComponent(dvInstance);
 return;
}

static void captureDeviceControlRewind(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
	
	ComponentResult err = QT::noErr;
	ComponentInstance dvInstance;
	
	ComponentDescription desc;
	desc.componentType = kIDHComponentType;
	desc.componentSubType = kIDHSubtypeDV;
	desc.componentManufacturer = NULL;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	
	Component dvComponent = FindNextComponent(NULL, &desc);

	if (!dvComponent)
		return;
	
	err = OpenAComponent(dvComponent, &dvInstance);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	QTAtomContainer deviceList;
	long numberDVDevices;
	
	err = IDHGetDeviceList(dvInstance, &deviceList);
	if (err != QT::noErr)
		{
		CloseComponent(dvInstance);
		captureFireErrorEvent(instance, err);
		return;
		}
	
	numberDVDevices = QTCountChildrenOfType(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType);
	if(numberDVDevices == 0)
		{
		CloseComponent(dvInstance);
		return;
		}

	for(long i=1; i <= numberDVDevices; ++i)
		{
		QTAtom deviceAtom; // get the atom to each device
		if (NULL == (deviceAtom = QTFindChildByIndex(deviceList, kParentAtomIsContainer, kIDHDeviceAtomType, i, NULL))){ continue; }
		
		QTAtom isochAtom; // find the isoch characteristics for this device
		if (NULL == (isochAtom = QTFindChildByIndex(deviceList, deviceAtom, kIDHIsochServiceAtomType, 1, NULL))){ continue; }
		
		long nConfigs = QTCountChildrenOfType( deviceList, isochAtom, kIDHIsochModeAtomType);
		
		for(long j = 1; j <= nConfigs; ++j)
			{
			QTAtom configAtom; // get this configs atom
			if (NULL == (configAtom = QTFindChildByIndex( deviceList, isochAtom, kIDHIsochModeAtomType, j, NULL))){ continue; }
			
			QTAtom mediaAtom; // find the media type atom
			if (NULL == (mediaAtom = QTFindChildByIndex(deviceList, configAtom, kIDHIsochMediaType, 1, NULL))){ continue; }
			
			QTLockContainer(deviceList);
			
			OSType mediaType = 0; // get the value of the mediaType atom
			Size siz;
			QTCopyAtomDataToPtr(deviceList, mediaAtom, true, sizeof mediaType, &mediaType, &siz);
			QTUnlockContainer(deviceList);
			
			if(kIDHVideoMediaAtomType == mediaType)
				{
				// found first appropriate media type.
				QTAtomSpec videoConfig;
				// found video device
				videoConfig.container = deviceList;
				videoConfig.atom = configAtom;
				
				err = IDHSetDeviceConfiguration(dvInstance, &videoConfig);
				if (err != QT::noErr)
					continue;
				
				ComponentInstance deviceControlInstance;
				
				err = IDHGetDeviceControl(dvInstance, &deviceControlInstance);
				if (err != QT::noErr)
					continue;

				
				char inBuffer[4];
				char outBuffer[16];
				DVCTransactionParams pParams;
				
				// fill up the avc frame
				inBuffer[0] = 0x00; //kAVCControlCommand;
				inBuffer[1] = 0x20;
				inBuffer[2] = 0xC4; // RWD command is 0xc4 0x65
				inBuffer[3] = 0x65;
				
				// fill up the transaction parameter block
				pParams.commandBufferPtr = inBuffer;
				pParams.commandLength = sizeof inBuffer;
				pParams.responseBufferPtr = outBuffer;
				pParams.responseBufferSize = sizeof outBuffer;
				pParams.responseHandler = NULL;
				std::memset(outBuffer, 0, sizeof outBuffer);
				
				err = DeviceControlDoAVCTransaction(deviceControlInstance, &pParams);
				if (err != QT::noErr)
					{
					CloseComponent(dvInstance);
					captureFireErrorEvent(instance, err);
					return;
					}
					
				
				CloseComponent(dvInstance);
				return;
				}
			}
		}
 // nothing acceptable found.
 CloseComponent(dvInstance);
 return;
}

static void captureDeviceControlRecord(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
}

static void captureDeviceControlRecordFreeze(REALcontrolInstance instance)
{
	// get our private data
	ControlData(captureControl, instance, captureData, data);
}



REALproperty captureProperties[] = {
	{ "Appearance", "PreviewPicture", "Picture", REALpropInvalidate, REALstandardGetter, REALstandardSetter, FieldOffset(captureData, theNoPreviewPicture)},
	{ "InitialState", "EnableVideo", "Boolean", 0, (REALproc) captureGetEnableVideo, (REALproc) captureSetEnableVideo, 0 },
	{ "InitialState", "EnableSound", "Boolean", 0, (REALproc) captureGetEnableSound, (REALproc) captureSetEnableSound, 0 },
	{ nil, "UserSettings", "String", REALpropRuntimeOnly, (REALproc) captureGetSettings, (REALproc) captureSetSettings, 0 },
	{ nil, "PreviewMode", "Integer", REALpropRuntimeOnly, (REALproc) captureGetPreviewMode, (REALproc) captureSetPreviewMode, 0 },
	//{ "Behavior", "Preview", "Boolean", 0, REALstandardGetter, REALstandardSetter, FieldOffset(captureData, previewState)},
	{ "Behavior", "Preview", "Boolean", 0, (REALproc) captureGetPreviewState, (REALproc) captureSetPreviewState, 0 },
	//{ "Behavior", "PreviewWhileRecording", "Boolean", 0, REALstandardGetter, REALstandardSetter, FieldOffset(captureData, previewWhileRecording)},
	{ "Behavior", "PreviewWhileRecording", "Boolean", 0, (REALproc) captureGetPreviewWhileRecordingState, (REALproc) captureSetPreviewWhileRecordingState, 0 },
	{ nil, "Pause", "Boolean", REALpropRuntimeOnly, (REALproc) captureGetPause, (REALproc) captureSetPause, 0 },
	{ nil, "LevelMeterActive", "Boolean", REALpropRuntimeOnly, (REALproc) captureGetMeterState, (REALproc) captureSetMeterState, 0 },
	{ nil, "AGCActive", "Boolean", REALpropRuntimeOnly, (REALproc) captureGetAGCState, (REALproc) captureSetAGCState, 0 },
	{ nil, "SoundInputGain", "Integer", REALpropRuntimeOnly, (REALproc) captureGetGain, (REALproc) captureSetGain, 0 },
	{ nil, "SoundVolume", "Integer", REALpropRuntimeOnly, (REALproc) captureGetSoundVolume, (REALproc) captureSetSoundVolume, 0 },
	{ nil, "SampleRate", "Integer", REALpropRuntimeOnly, (REALproc) captureGetSampleRate, (REALproc) captureSetSampleRate, 0 },
	{ nil, "SampleSize", "Integer", REALpropRuntimeOnly, (REALproc) captureGetSampleSize, (REALproc) captureSetSampleSize, 0 },
	{ nil, "SoundChannelCount", "Integer", REALpropRuntimeOnly, (REALproc) captureGetNumChan, (REALproc) captureSetNumChan, 0 },
	{ nil, "SoundCompressionType", "String", REALpropRuntimeOnly, (REALproc) captureGetSoundCompressionType, (REALproc) captureSetSoundCompressionType, 0 },
	{ nil, "VideoCompressionType", "String", REALpropRuntimeOnly, (REALproc) captureGetVideoCompressionType, (REALproc) captureSetVideoCompressionType, 0 },
	{ nil, "VideoBlackLevel", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoBlackLevelValue, (REALproc) captureSetVideoBlackLevelValue, 0 }, // BJS 11/12/02
	{ nil, "VideoBrightness", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoBrightness, (REALproc) captureSetVideoBrightness, 0 }, // BJS 11/12/02
	{ nil, "VideoContrast", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoContrast, (REALproc) captureSetVideoContrast, 0 }, // BJS 11/12/02
	{ nil, "VideoHue", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoHue, (REALproc) captureSetVideoHue, 0 }, // BJS 11/12/02
	{ nil, "VideoSaturation", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSaturation, (REALproc) captureSetVideoSaturation, 0 }, // BJS 11/12/02
	{ nil, "VideoSharpness", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSharpness, (REALproc) captureSetVideoSharpness, 0 }, // BJS 11/12/02
	{ nil, "VideoWhiteLevel", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoWhiteLevelValue, (REALproc) captureSetVideoWhiteLevelValue, 0 }, // BJS 11/12/02
	{ nil, "FrameRate", "Double", REALpropRuntimeOnly, (REALproc) captureGetFrameRate, (REALproc) captureSetFrameRate, 0 },
	{ nil, "KeyFrameRate", "Integer", REALpropRuntimeOnly, (REALproc) captureGetKeyFrameRate, (REALproc) captureSetKeyFrameRate, 0 },
	{ nil, "VideoDepth", "Integer", REALpropRuntimeOnly, (REALproc) captureGetDepth, (REALproc) captureSetDepth, 0 },
	{ nil, "SpatialQuality", "Integer", REALpropRuntimeOnly, (REALproc) captureGetSpatialQuality, (REALproc) captureSetSpatialQuality, 0 },
	{ nil, "TemporalQuality", "Integer", REALpropRuntimeOnly, (REALproc) captureGetTemporalQuality, (REALproc) captureSetTemporalQuality, 0 },
	{ nil, "VideoWidth", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoWidth },
	{ nil, "VideoHeight", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoHeight },
	{ nil, "FastPictureGrab", "Boolean", REALpropRuntimeOnly, (REALproc) captureGetFastGrabPicture, (REALproc) captureSetFastGrabPicture, 0 },
	
	//NEW PROPS - WE NEED TO ADD THESE TO MAC SIDE TOO
	{ nil, "VideoBrightnessMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoBrightnessMax },
	{ nil, "VideoBrightnessMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoBrightnessMin },
	{ nil, "VideoContrastMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoContrastMax },
	{ nil, "VideoContrastMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoContrastMin },
	{ nil, "VideoHueMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoHueMax },
	{ nil, "VideoHueMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoHueMin },
	{ nil, "VideoSaturationMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSaturationMax },
	{ nil, "VideoSaturationMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSaturationMin },
	{ nil, "VideoSharpnessMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSharpnessMax },
	{ nil, "VideoSharpnessMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoSharpnessMin },
	{ nil, "VideoWhiteLevelMax", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoWhiteLevelMax },
	{ nil, "VideoWhiteLevelMin", "Integer", REALpropRuntimeOnly, (REALproc) captureGetVideoWhiteLevelMin },

};

REALmethodDefinition captureMethods[] = {
	{ (REALproc) captureShowVideoSettingsDialog, REALnoImplementation, "ShowVideoSettings() As Boolean" },
//	{ (REALproc) captureShowVideoSettingsPinDialog, REALnoImplementation, "ShowVideoPinSettings() As Boolean" },

	{ (REALproc) captureGetSoundDeviceCount, REALnoImplementation, "SoundDeviceCount() As Integer" },
	{ (REALproc) captureGetIndexedSoundDeviceName, REALnoImplementation, "SoundDevice(index As Integer) As String" },
	{ (REALproc) captureGetIndexedSoundDeviceAvailable, REALnoImplementation, "SoundDeviceAvailable(index As Integer) As Boolean" },
	{ (REALproc) captureGetSoundDeviceCurrent, REALnoImplementation, "SelectedSoundDevice() As Integer" },
	{ (REALproc) captureSetSoundDeviceName, REALnoImplementation, "SetSoundDevice(name As String)" },
	{ (REALproc) captureGetCurrentSoundDeviceSourceCount, REALnoImplementation, "SelectedSoundDeviceInputCount() As Integer" },
	{ (REALproc) captureGetIndexedSoundDeviceSource, REALnoImplementation, "SoundInput(index As Integer) As String" },
	{ (REALproc) captureGetLeftLevel, REALnoImplementation, "LeftSoundLevelMeter() As Integer" },
	{ (REALproc) captureGetRightLevel, REALnoImplementation, "RightSoundLevelMeter() As Integer" },
	{ (REALproc) captureSetSoundDeviceSourceIndex, REALnoImplementation, "SetSoundInput(index As Integer)" },
	{ (REALproc) captureGetCurrentSoundDeviceSource, REALnoImplementation, "SelectedSoundInput() As Integer" },
	{ (REALproc) captureGetVideoDeviceCount, REALnoImplementation, "VideoDeviceCount() As Integer" },
	{ (REALproc) captureGetIndexedVideoDeviceName, REALnoImplementation, "VideoDevice(index As Integer) As String" },
	{ (REALproc) captureGetIndexedVideoDeviceAvailable, REALnoImplementation, "VideoDeviceAvailable(index As Integer) As Boolean" },
	{ (REALproc) captureGetVideoDeviceCurrent, REALnoImplementation, "SelectedVideoDevice() As Integer" },
	{ (REALproc) captureSetVideoDeviceName, REALnoImplementation, "SetVideoDevice(name As String)" },

	{ (REALproc) captureSetVideoDeviceSourceIndex, REALnoImplementation, "SetVideoInput(index As Integer)" },

	{ (REALproc) captureShowSoundSettingsDialog, REALnoImplementation, "ShowSoundSettings() As Boolean" },
	{ (REALproc) captureIsRecording, REALnoImplementation, "Recording() As Boolean" },
	{ (REALproc) captureStartRecordingAsync, REALnoImplementation, "RecordAsync(theMovieFile As FolderItem)" },
	{ (REALproc) captureStartRecordingAsyncTime, REALnoImplementation, "RecordAsync(theMovieFile As FolderItem, recordTicks As Integer)" },
	{ (REALproc) captureStartRecordingSync, REALnoImplementation, "RecordSync(theMovieFile As FolderItem)" },
	{ (REALproc) captureStartRecordingSyncTime, REALnoImplementation, "RecordSync(theMovieFile As FolderItem, recordTicks As Integer)" },
	{ (REALproc) captureStop, REALnoImplementation, "Stop()" },
	{ (REALproc) captureGrabPict, REALnoImplementation, "GrabPicture() As Picture" },
	
	
	{ (REALproc) captureDeviceControlPlay, REALnoImplementation, "DeviceControlPlay()" },
	{ (REALproc) captureDeviceControlStop, REALnoImplementation, "DeviceControlStop()" },
	{ (REALproc) captureDeviceControlFreeze, REALnoImplementation, "DeviceControlPause()" },
	{ (REALproc) captureDeviceControlFastForward, REALnoImplementation, "DeviceControlFastForward()" },
	{ (REALproc) captureDeviceControlRewind, REALnoImplementation, "DeviceControlRewind()" },
	{ (REALproc) captureDeviceControlRecord, REALnoImplementation, "DeviceControlRecord()" },
	{ (REALproc) captureDeviceControlRecordFreeze, REALnoImplementation, "DeviceControlRecordPause()" },


	{ (REALproc) captureDeviceControlPoll, REALnoImplementation, "Poll()" },

	
};

REALcontrolBehaviour captureBehaviour = {
	captureInit, 
	captureDispose,
	captureDraw, //the method called to redraw the control
	nil,
	nil,
	nil,
	nil,
	nil,
	nil,
	captureOpen,
	captureClose
};

REALcontrol captureControl = {
	kCurrentREALControlVersion,
	"BESTCapture",                // name of control
	sizeof(captureData),
	0,					// for invisible controls like Timer, use: REALinvisibleControl
	128,				//PICT resource for toolbar 
	129,				//PICT resource for toolbar (depressed)
	320, 240,			//the default size of the control when dragged to a window
	captureProperties,
	sizeof(captureProperties) / sizeof(REALproperty),
	captureMethods,
	sizeof(captureMethods) / sizeof(REALmethodDefinition),
	captureEvents,
	sizeof(captureEvents) / sizeof(REALevent),
	&captureBehaviour
};

static REALstring gCaptureGetIDPath(int index)
{
	#pragma unused(index)
	if (REALinRuntime())
	{
	return NULL;
	}
	return NULL;
}

REALmethodDefinition gCaptureGetIDDef = {(REALproc) gCaptureGetIDPath, REALnoImplementation, "BESTGetDevicePath(index As Integer) As String" };

void PluginEntry(void)
{
	REALRegisterControl(&captureControl);
	REALRegisterMethod(&gCaptureGetIDDef);
}
